# -*- coding:utf-8 -*-

from django.conf import settings
from django.conf.urls import patterns, include, url
from django.views.generic.detail import DetailView
from coop_cms.settings import get_article_class, get_article_views, keep_deprecated_func_views_for_article

urlpatterns = patterns('coop_cms.views',
    url(r'^cms/tree/(?P<tree_id>\d*)/$', 'process_nav_edition', name='navigation_tree'),
    url(r'^cms/media-images/$', 'show_media', {'media_type': 'image'}, name='coop_cms_media_images'),
    url(r'^cms/media-documents/$', 'show_media', {'media_type': 'document'}, name='coop_cms_media_documents'),
    url(r'^cms/upload-image/$', 'upload_image', name="coop_cms_upload_image"),
    url(r'^cms/upload-doc/$', 'upload_doc', name="coop_cms_upload_doc"),
    url(r'^cms/change-template/(?P<article_id>\d*)/$', 'change_template', name="coop_cms_change_template"),
    url(r'^cms/settings/(?P<article_id>\d*)/$', 'article_settings', name="coop_cms_article_settings"),
    url(r'^cms/new/$', 'new_article', name="coop_cms_new_article"),
    url(r'^cms/new/article/$', 'new_article', name="coop_cms_new_article"),
    url(r'^cms/new/link/$', 'new_link', name="coop_cms_new_link"),
    url(r'^cms/update-logo/(?P<article_id>\d*)/$', 'update_logo', name="coop_cms_update_logo"),
    url(r'^cms/private-download/(?P<doc_id>\d*)/$', 'download_doc', name='coop_cms_download_doc'),
    url(r'^cms/articles/$', 'view_all_articles', name="coop_cms_view_all_articles"),
    url(r'^cms/$', 'view_all_articles'),
    url(r'^cms/set-homepage/(?P<article_id>\d*)/$', 'set_homepage', name='coop_cms_set_homepage'),
    url(r'^cms/newsletter/new/$', 'new_newsletter', name='coop_cms_new_newsletter'),
    url(r'^cms/newsletter/settings/(?P<newsletter_id>\d+)/$', 'new_newsletter', name='coop_cms_newsletter_settings'),
    url(r'^cms/newsletter/(?P<newsletter_id>\d+)/$', 'view_newsletter', name='coop_cms_view_newsletter'),
    url(r'^cms/newsletter/(?P<newsletter_id>\d+)/cms_edit/$', 'edit_newsletter', name='coop_cms_edit_newsletter'),
    url(r'^cms/newsletter/change-template/(?P<newsletter_id>\d+)/$', 'change_newsletter_template', name="coop_cms_change_newsletter_template"),
    url(r'^cms/newsletter/test/(?P<newsletter_id>\d+)/$', 'test_newsletter', name="coop_cms_test_newsletter"),
    url(r'^cms/newsletter/schedule/(?P<newsletter_id>\d+)/$', 'schedule_newsletter_sending', name="coop_cms_schedule_newsletter_sending"),
    url(r'sitemap/$', 'tree_map', name="default_site_map"),
    url(r'articles/(?P<slug>[-\w]+)/$', 'articles_category', name="coop_cms_articles_category"),
    url(r'cms/change-language/$', 'change_language', name='coop_cms_change_language'),
)

if 'coop_cms.apps.rss_sync' in settings.INSTALLED_APPS:
    urlpatterns += patterns('',
        (r'^rss-sync/', include('coop_cms.apps.rss_sync.urls')),
    )
    
if 'coop_cms.apps.test_app' in settings.INSTALLED_APPS:
    urlpatterns += patterns('',
        (r'^this-should-be-only-in-test-mode', include('coop_cms.apps.test_app.urls')),
    )

urlpatterns += patterns('coop_cms.views',
    #keep these at the end
    url(r'^(?P<url>.+)/cms_publish/$', 'publish_article', name='coop_cms_publish_article'),
    url(r'^(?P<url>.+)/cms_cancel/$', 'cancel_edit_article', name='coop_cms_cancel_edit_article'),
    url(r'^$', 'homepage', name='coop_cms_homepage'),
)

if keep_deprecated_func_views_for_article():
    urlpatterns += patterns('coop_cms.views',
        url(r'^(?P<url>.+)/cms_edit/$', 'edit_article', name='coop_cms_edit_article'),
        url(r'^(?P<url>.+)/$', 'view_article', name='coop_cms_view_article'),
    )
else:
    article_views = get_article_views()    
    ArticleView = article_views['article_view']
    EditArticleView = article_views['edit_article_view']
    urlpatterns += patterns('',
        url(r'^(?P<slug>.+)/cms_edit/$', EditArticleView.as_view(), name='coop_cms_edit_article'),
        url(r'^(?P<slug>.+)/$', ArticleView.as_view(), name='coop_cms_view_article'),
    )

#urlpatterns += patterns('coop_cms.views',
#    #keep these at the end
#    url(r'^(?P<url>.+)/cms_publish/$', 'publish_article', name='coop_cms_publish_article'),
#    url(r'^(?P<url>.+)/cms_edit/$', 'edit_article', name='coop_cms_edit_article'),
#    url(r'^(?P<url>.+)/cms_cancel/$', 'cancel_edit_article', name='coop_cms_cancel_edit_article'),
#    url(r'^(?P<url>.+)/$', 'view_article', name='coop_cms_view_article'),
#    url(r'^$', 'homepage', name='coop_cms_homepage'),
#)



# -*- coding: utf-8 -*-

from django.views.generic.list import ListView as DjangoListView
from django.views.generic.base import View
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from djaloha import utils as djaloha_utils
from django.contrib.messages.api import success as success_message
from django.contrib.messages.api import error as error_message
from django.utils.translation import ugettext as _
from django.http import HttpResponse, Http404, HttpResponseRedirect, HttpResponseForbidden
from django.core.exceptions import ValidationError, PermissionDenied
import logging
logger = logging.getLogger("coop_cms")

class ListView(DjangoListView):
    ordering = ''
    
    def get_context_data(self, **kwargs):
        context = super(ListView, self).get_context_data(**kwargs)
        context['model'] = self.model
        return context
    
    def get_queryset(self):
        if self.ordering:
            if type(self.ordering) in (list, tuple):
                return self.model.objects.order_by(*self.ordering)
            else:
                return self.model.objects.order_by(self.ordering)
        else:
            return self.model.objects.all()


class EditableObjectView(View):
    model = None
    template_name = ""
    form_class = None
    field_lookup = "pk"
    edit_mode = False
    varname = "object"
    
    def __init__(self, *args, **kwargs):
        super(EditableObjectView, self).__init__(*args, **kwargs)
    
    def can_edit_object(self):
        can_edit_perm = 'can_edit_{0}'.format(self.varname)
        return self.request.user.is_authenticated() and self.request.user.has_perm(can_edit_perm, self.object)
        
    def can_view_object(self):
        if self.edit_mode:
            return self.can_edit_object()
        else:
            can_view_perm = 'can_view_{0}'.format(self.varname)
            return self.request.user.has_perm(can_view_perm, self.object)
    
    def get_object(self):
        lookup = {self.field_lookup: self.kwargs[self.field_lookup]}
        return get_object_or_404(self.model, **lookup)
    
    def get_context_data(self):
        return {
            'form': self.form if self.edit_mode else None,
            'editable': self.can_edit_object(),
            'edit_mode': self.edit_mode,
            'title': getattr(self.object, 'title', unicode(self.object)),
            self.varname: self.object,
        }
        
    def get_template(self):
        return self.template_name
    
    def handle_object_not_found(self):
        pass
    
    def get(self, request, *args, **kwargs):
        try:
            self.object = self.get_object()
        except Http404:
            return_this = self.handle_object_not_found()
            if return_this:
                return return_this
            else:
                raise
        
        if not self.can_view_object():
            logger.error("PermissionDenied")
            error_message(request, _(u'Permission denied'))
            raise PermissionDenied
        
        self.form = self.form_class(instance=self.object)
        
        return render_to_response(
            self.get_template(),
            self.get_context_data(),
            context_instance=RequestContext(request)
        )
    
    def after_save(self, object):
        pass
    
    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        
        if not self.can_edit_object():
            logger.error("PermissionDenied")
            error_message(request, _(u'Permission denied'))
            raise PermissionDenied
        
        self.form = self.form_class(request.POST, request.FILES, instance=self.object)

        forms_args = djaloha_utils.extract_forms_args(request.POST)
        djaloha_forms = djaloha_utils.make_forms(forms_args, request.POST)

        if self.form.is_valid() and all([f.is_valid() for f in djaloha_forms]):
            self.object = self.form.save()
            
            self.after_save(self.object)
            
            if djaloha_forms:
                [f.save() for f in djaloha_forms]

            success_message(request, _(u'The object has been saved properly'))

            return HttpResponseRedirect(self.object.get_absolute_url())
        else:
            error_text = u'<br />'.join([unicode(f.errors) for f in [self.form]+djaloha_forms if f.errors])
            error_message(request, _(u'An error occured: {0}'.format(error_text)))
            logger.debug("error: error_text")
    
        return render_to_response(
            self.get_template(),
            self.get_context_data(),
            context_instance=RequestContext(request)
        )




# -*- coding: utf-8 -*-
from django.contrib import admin
from coop_cms.admin import ArticleAdmin as CmsArticleAdmin

class ArticleAdmin(CmsArticleAdmin):
    fieldsets = CmsArticleAdmin.fieldsets + (
        ('Misc', {'fields': ('author',)}),
    )

from coop_cms.settings import get_article_class
admin.site.unregister(get_article_class())
admin.site.register(get_article_class(), ArticleAdmin)



#-*- coding: utf-8 -*-

from django import forms
from coop_cms.apps.rss_sync import models, widgets
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

#These 2 admin forms makes possible to add custom widgets for the id field

class RssSourceAdminForm(forms.ModelForm):
    url = forms.CharField(widget=widgets.AdminCollectRssWidget)
    class Meta:
        model = models.RssSource
        
    class Media:
        css = {
            'all': (settings.STATIC_URL+'css/rss_sync/admin-cust.css',),
        }
        
        
class RssItemAdminForm(forms.ModelForm):
    id = forms.IntegerField(widget=widgets.AdminCreateArticleWidget, label=u'Id')
    class Meta:
        model = models.RssItem



# -*- coding: utf-8 -*-

from django.conf import settings

def cms_settings(request):
    context = {}
    
    if hasattr(settings, 'COOP_CMS_JQUERY_VERSION'):
        context['COOP_CMS_JQUERY_VERSION'] = settings.COOP_CMS_JQUERY_VERSION
    
    return context
    


# -*- coding: utf-8 -*-

from django.core.management.base import BaseCommand
from datetime import datetime
from coop_cms.utils import send_newsletter
from coop_cms.models import NewsletterSending

class Command(BaseCommand):
    help = u"send newsletter"

    def handle(self, *args, **options):
        #look for emailing to be sent
        verbose = options.get('verbosity', 1)
        
        if len(args)>0:
            dests = args[0].split(";")
        else:
            print 'usage: python manage.py send_newsletter toto@toto.fr;titi@titi.fr'
        
        sendings = NewsletterSending.objects.filter(scheduling_dt__lte=datetime.now(), sending_dt=None)
        for sending in sendings:
            if verbose:
                print 'send_newsletter {1} to {0} addresses'.format(len(dests), sending.newsletter)
            
            nb_sent = send_newsletter(sending.newsletter, dests)
            
            if verbose:
                print nb_sent, "emails sent"
            
            sending.sending_dt = datetime.now()
            sending.save()
        


# -*- coding: utf-8 -*-

import floppyforms as forms
from djaloha.widgets import AlohaInput
from models import TestClass

class TestClassForm(forms.ModelForm):
    
    class Meta:
        model = TestClass
        fields = ('field1', 'field2',)
        widgets = {
            'field1': AlohaInput(),
            'field2': AlohaInput(),
        }


# -*- coding: utf-8 -*-

from django.test import TestCase
from django.contrib.auth.models import User, Permission
from django.core.urlresolvers import reverse
from django.template import Template, Context
from model_mommy import mommy
from django.conf import settings
from models import TestClass
import logging

class GenericViewTestCase(TestCase):
    
    def setUp(self):
        logging.disable(logging.CRITICAL)
        
    def tearDown(self):
        logging.disable(logging.NOTSET)

    def _log_as_viewer(self):
        self.viewer = user = User.objects.create_user('viewer', 'viewer@toto.fr', 'viewer')
        return self.client.login(username='viewer', password='viewer')
        
    def _log_as_editor(self):
        self.editor = User.objects.create_user('editor', 'toto@toto.fr', 'editor')
        self.editor.is_staff = True
        self.editor.save()
        return self.client.login(username='editor', password='editor')
    
    def test_view_list_objects(self):
        obj = mommy.make(TestClass)
        response = self.client.get(obj.get_list_url())
        self.assertEqual(200, response.status_code)
    
    def test_view_object_anomymous(self):
        obj = mommy.make(TestClass)
        response = self.client.get(obj.get_absolute_url())
        self.assertEqual(403, response.status_code)
    
    def test_edit_object_anonymous(self):
        obj = mommy.make(TestClass)
        response = self.client.get(obj.get_edit_url())
        self.assertEqual(403, response.status_code)
        
        data = {'field1': "ABC", 'field2': "DEF"}
        response = self.client.post(obj.get_edit_url(), data=data, follow=True)
        self.assertEqual(403, response.status_code)
        
        obj = TestClass.objects.get(id=obj.id)
        self.assertEqual(obj.field1, "")
        self.assertEqual(obj.field2, "")
        
    def test_view_object_viewer(self):
        self._log_as_viewer()
        obj = mommy.make(TestClass)
        response = self.client.get(obj.get_absolute_url())
        self.assertEqual(200, response.status_code)
    
    def test_edit_object_viewer(self):
        self._log_as_viewer()
        obj = mommy.make(TestClass)
        response = self.client.get(obj.get_edit_url())
        self.assertEqual(403, response.status_code)
        
        data = {'field1': "ABC", 'field2': "DEF"}
        response = self.client.post(obj.get_edit_url(), data=data, follow=True)
        self.assertEqual(403, response.status_code)
        
        obj = TestClass.objects.get(id=obj.id)
        self.assertEqual(obj.field1, "")
        self.assertEqual(obj.field2, "")
        
    def test_view_object_editor(self):
        self._log_as_editor()
        obj = mommy.make(TestClass)
        response = self.client.get(obj.get_absolute_url())
        self.assertEqual(200, response.status_code)
    
    def test_edit_object_editor(self):
        self._log_as_editor()
        obj = mommy.make(TestClass)
        response = self.client.get(obj.get_edit_url())
        self.assertEqual(200, response.status_code)
        
        data = {'field1': "ABC", 'field2': "DEF"}
        response = self.client.post(obj.get_edit_url(), data=data, follow=True)
        self.assertEqual(200, response.status_code)
        
        obj = TestClass.objects.get(id=obj.id)
        self.assertEqual(obj.field1, data["field1"])
        self.assertEqual(obj.field2, data["field2"])
        
        



# -*- coding: utf-8 -*-

from models import TestClass
from forms import TestClassForm

from coop_cms.generic_views import EditableObjectView, ListView

class TestClassListView(ListView):
    model = TestClass
    template_name = "coop_cms/test_app/list.html"
    ordering = 'other_field'

class TestClassDetailView(EditableObjectView):
    model = TestClass
    edit_mode = False
    form_class = TestClassForm
    template_name = "coop_cms/test_app/detail.html"
    
class TestClassEditView(TestClassDetailView):
    edit_mode = True
    


# -*- coding: utf-8 -*-

from django import template
from django.template.loader import get_template
from django.conf import settings
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _
from coop_cms.models import NavNode
from coop_cms.settings import get_navtree_class
from django.contrib.contenttypes.models import ContentType
register = template.Library()
from django.template import VariableDoesNotExist


def extract_kwargs(args):
    kwargs = {}
    for arg in args:
        try:
            key, value = arg.split('=')
            kwargs[key] = value
        except ValueError:  # No = in the arg
            pass
    return kwargs


class NavigationTemplateNode(template.Node):
    def __init__(self, *args, **kwargs):
        super(NavigationTemplateNode, self).__init__()
        self._kwargs = {}
        for (k, v) in kwargs.items():
            self._kwargs[k] = template.Variable(v)

    def format_css_class(self, class_name):
        return u' class="{0}"'.format(class_name) if class_name else u""

    def resolve_kwargs(self, context):
        kwargs = {}
        for (k, v) in self._kwargs.items():
            try:
                kwargs[k] = v.resolve(context)
            except VariableDoesNotExist:
                kwargs[k] = v.var  # if the variable can not be resolved, thake the value as is

            if k == 'css_class':
                kwargs[k] = self.format_css_class(v)

        if not 'tree' in kwargs:
            kwargs['tree'] = 'default'

        tree, _is_new = get_navtree_class().objects.get_or_create(name=kwargs['tree'])
        if 'coop_cms_navtrees' in context.dicts[0]:
            context.dicts[0]['coop_cms_navtrees'].append(tree)
        else:
            context.dicts[0]['coop_cms_navtrees'] = [tree]

        return kwargs

#----------------------------------------------------------


class NavigationAsNestedUlNode(NavigationTemplateNode):

    def __init__(self, **kwargs):
        super(NavigationAsNestedUlNode, self).__init__(**kwargs)

    def render(self, context):
        kwargs = self.resolve_kwargs(context)
        tree_name = kwargs.pop('tree', 'default')
        root_nodes = NavNode.objects.filter(tree__name=tree_name, parent__isnull=True).order_by("ordering")
        return u''.join([node.as_navigation(**kwargs) for node in root_nodes])


@register.tag
def navigation_as_nested_ul(parser, token):
    args = token.contents.split()
    kwargs = extract_kwargs(args)
    return NavigationAsNestedUlNode(**kwargs)

#----------------------------------------------------------


class NavigationBreadcrumbNode(NavigationTemplateNode):
    def __init__(self, object, **kwargs):
        super(NavigationBreadcrumbNode, self).__init__(**kwargs)
        self.object_var = template.Variable(object)

    def render(self, context):
        object = self.object_var.resolve(context)
        ct = ContentType.objects.get_for_model(object.__class__)
        kwargs = self.resolve_kwargs(context)
        tree_name = kwargs.pop('tree', 'default')
        nav_nodes = NavNode.objects.filter(tree__name=tree_name, content_type=ct, object_id=object.id)
        if nav_nodes.count() > 0:
            return nav_nodes[0].as_breadcrumb(**kwargs)
        return u''


@register.tag
def navigation_breadcrumb(parser, token):
    args = token.contents.split()
    kwargs = extract_kwargs(args)
    if len(args) < 2:
        raise template.TemplateSyntaxError(_("navigation_breadcrumb requires object as argument"))
    return NavigationBreadcrumbNode(args[1], **kwargs)


class NavigationChildrenNode(NavigationTemplateNode):

    def __init__(self, object, **kwargs):
        super(NavigationChildrenNode, self).__init__(**kwargs)
        self.object_var = template.Variable(object)

    def render(self, context):
        object = self.object_var.resolve(context)
        ct = ContentType.objects.get_for_model(object.__class__)
        kwargs = self.resolve_kwargs(context)
        tree_name = kwargs.pop('tree', 'default')
        nav_nodes = NavNode.objects.filter(tree__name=tree_name, content_type=ct, object_id=object.id)
        if nav_nodes.exists():
            return nav_nodes[0].children_as_navigation(**kwargs)
        return u''


@register.tag
def navigation_children(parser, token):
    args = token.contents.split()
    kwargs = extract_kwargs(args)
    if len(args) < 2:
        raise template.TemplateSyntaxError(_("navigation_children requires object as argument and optionally tree={{tree_name}}"))
    return NavigationChildrenNode(args[1], **kwargs)


class NavigationSiblingsNode(NavigationTemplateNode):

    def __init__(self, object, **kwargs):
        super(NavigationSiblingsNode, self).__init__(**kwargs)
        self.object_var = template.Variable(object)

    def render(self, context):
        object = self.object_var.resolve(context)
        ct = ContentType.objects.get_for_model(object.__class__)
        kwargs = self.resolve_kwargs(context)
        tree_name = kwargs.pop('tree', 'default')
        nav_nodes = NavNode.objects.filter(tree__name=tree_name, content_type=ct, object_id=object.id)
        if nav_nodes.count() > 0:
            return nav_nodes[0].siblings_as_navigation(**kwargs)
        return u''


@register.tag
def navigation_siblings(parser, token):
    args = token.contents.split()
    kwargs = extract_kwargs(args)
    if len(args) < 2:
        raise template.TemplateSyntaxError(_("navigation_siblings requires object as argument"))
    return NavigationSiblingsNode(args[1], **kwargs)



# -*- coding:utf-8 -*-
from django.conf.urls import url
from django.contrib.sitemaps import Sitemap
from coop_cms.settings import get_article_class
from coop_cms.models import BaseArticle
from django.conf import settings

class ArticleSitemap(Sitemap):
    changefreq = "weekly"
    priority = 0.5

    def items(self):
        Article = get_article_class()
        return Article.objects.filter(publication=BaseArticle.PUBLISHED)

    def lastmod(self, obj):
        return obj.modified

def get_sitemaps(langs=None):
    if 'localeurl' in settings.INSTALLED_APPS:
        from localeurl.sitemaps import LocaleurlSitemap
        class LocaleArticleSitemap(LocaleurlSitemap, ArticleSitemap): pass
        sitemaps = {}
        lang_codes = langs or [code for (code, _x) in settings.LANGUAGES]
        for code in lang_codes:
            sitemaps['coop_cms_'+code] = LocaleArticleSitemap(code) 
    else:
        sitemaps = {
            'coop_cms': ArticleSitemap,
        }
    return sitemaps
    
urlpatterns = (
    url(r'^sitemap\.xml$', 'django.contrib.sitemaps.views.sitemap', {'sitemaps': get_sitemaps()}),
)


# -*- coding: utf-8 -*-

from coop_cms.settings import get_article_class
from coop_cms.utils import get_article
from django import template
register = template.Library()
from django.template.defaultfilters import slugify
from coop_cms.utils import dehtml as do_dehtml
from bs4 import BeautifulSoup
import unicodedata
    
################################################################################
class ArticleLinkNode(template.Node):

    def __init__(self, title, lang):
        self.title = title
        self.lang = lang

    def render(self, context):
        Article = get_article_class()
        
        try:
            v = template.Variable(self.title)
            title = v.resolve(context)
        except template.VariableDoesNotExist:
            title = self.title.strip("'").strip('"')
        
        slug = slugify(title)
        try:
            if self.lang:
                article = get_article(slug, force_lang=self.lang)
            else:
                #If the language is not defined, we need to get it from the context
                #The get_language api doesn't work in templatetag
                request = context['request']
                article = get_article(slug, current_lang=request.LANGUAGE_CODE)
            
        except Article.DoesNotExist:
            article = Article.objects.create(slug=slug, title=title)
        
        return article.get_absolute_url()

@register.tag
def article_link(parser, token):
    args = token.split_contents()
    title = args[1]
    lang = args[2] if len(args) > 2 else None
    return ArticleLinkNode(title, lang)

@register.filter
def dehtml(value):
    return do_dehtml(value)
    
@register.filter
def sp_rt_lb(value):
    return value.replace("\n", " ").replace("\r", "")
    
################################################################################
class NewsletterFriendlyCssNode(template.Node):

    def __init__(self, nodelist_content, css):
        self.css = css
        self.nodelist_content = nodelist_content

    def render(self, context):
        content = self.nodelist_content.render(context)
        if context.get('by_email', False):
            soup = BeautifulSoup(content)
            for tag, css in self.css.items():
                for html_tag in soup.select(tag):
                    html_tag["style"] = css
            content = soup.prettify(formatter="minimal")
        else:
            style = ""
            for tag, value in self.css.items():
                style += u"{0} {{ {1} }}\n".format(tag, value)
            content = u"<style>\n{0}</style>\n".format(style) + content
        return content

@register.tag
def nlf_css(parser, token):
    #Newsletter friendly CSS
    args = token.split_contents()
    css = {}
    for item in args[1:]:
        tag, value = item.split("=")
        tag, value = tag.strip('"'), value.strip('"')
        css[tag] = value
    nodelist = parser.parse(('end_nlf_css',))
    token = parser.next_token()
    return NewsletterFriendlyCssNode(nodelist, css)

@register.filter
def normalize_utf8_to_ascii(ustr):
    try:
        return unicodedata.normalize('NFKD', ustr).encode('ascii','ignore')
    except TypeError:
        return ustr
    


from coop_cms.coop_bar_cfg import *

def load_commands(coop_bar):

    coop_bar.register([
        #[django_admin, django_admin_edit_article],
        [edit_newsletter, cancel_edit_newsletter, save_newsletter,
            change_newsletter_settings, change_newsletter_template,
            test_newsletter, schedule_newsletter],
        [cms_edit, cms_view, cms_save, cms_cancel],
        [cms_new_article, cms_article_settings],
        [cms_publish],
        [cms_media_library, cms_upload_image, cms_upload_doc],
        [log_out]
    ])




# -*- coding: utf-8 -*-

from modeltranslation.translator import translator, TranslationOptions
from coop_cms.models import NavNode, ArticleCategory, PieceOfHtml

class PieceOfHtmlTranslationOptions(TranslationOptions):
    fields = ('content',)

translator.register(PieceOfHtml, PieceOfHtmlTranslationOptions)


class NavNodeTranslationOptions(TranslationOptions):
    fields = ('label',)

translator.register(NavNode, NavNodeTranslationOptions)


class ArticleCategoryTranslationOptions(TranslationOptions):
    fields = ('name',)

translator.register(ArticleCategory, ArticleCategoryTranslationOptions)



# -*- coding:utf-8 -*-

from django.core.cache import cache
from django import template

register = template.Library()

from coop_cms.settings import get_article_class


@register.tag
def last_articles(parser, token):
    try:
        args = token.split_contents()
        if len(args) == 4:
            tag_name, number, template, category = token.split_contents()
            return ArticleListNode(number, template, category)
        elif len(args) == 3:
            tag_name, number, template = token.split_contents()
            return ArticleListNode(number, template, None)
    except ValueError:
        raise template.TemplateSyntaxError('%s tag requires at least 2 arguments' % token.split_contents()[0])

def resolve(var_or_value, ctx):
    if var_or_value[0] == '"':
        return var_or_value[1:-1]
    return ctx.resolve_variable(var_or_value)

class ArticleListNode(template.Node):
    """
    This templatetag takes as arguments :
    - A number limit of articles #TODO pagination
    - A template to render each articles
    - An optional ArticleCategory object
    """
    def __init__(self, number, templ, category):
        self.number = number
        self.templ = templ
        if category:
            self.category = template.Variable(category)

    def last_articles(self, context):
        article_list = []
        all_articles = get_article_class().objects.all().order_by('-created')
        if hasattr(self, 'category'):
            category = self.category.resolve(context)
            all_articles = all_articles.filter(category=category)
        all_articles = all_articles[:self.number]
        for a in all_articles:
            if a.navigation_parent == None:
                article_list.append(a)
        return article_list

    def render(self, context):
        tmpl = resolve(self.templ, context)
        t = template.loader.get_template(tmpl)
        return ''.join([t.render(template.Context({ 'item': item })) for item in self.last_articles(context)])



# -*- coding: utf-8 -*-
from django.db import models
from django.db.models import Q
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.template.loader import get_template
from django.template import Context
from django_extensions.db.models import TimeStampedModel, AutoSlugField
from django.conf import settings
from sorl.thumbnail import default as sorl_thumbnail
import os, os.path, shutil
from django.core.urlresolvers import reverse
from django.db.models.aggregates import Max
from django.utils.html import escape
from django.core.exceptions import ValidationError
# from html_field.db.models import HTMLField
# from html_field import html_cleaner
from coop_cms.settings import get_article_class, get_article_logo_size, get_newsletter_item_classes
from coop_cms.settings import get_navtree_class, is_localized, COOP_CMS_NAVTREE_CLASS, get_article_templates
from coop_cms.settings import get_default_logo
from django.contrib.staticfiles import finders
from django.core.files import File
from django.db.models.signals import pre_delete, post_save
from django.template.defaultfilters import slugify, escape
from datetime import datetime
from django.utils import translation
import urlparse
from sorl.thumbnail import default
from django.contrib.sites.models import Site
ADMIN_THUMBS_SIZE = '60x60'


def get_object_label(content_type, object):
    """
    returns the label used in navigation according to the configured rule
    """
    if not object:
        return _(u"Node")
    try:
        nt = NavType.objects.get(content_type=content_type)
        if nt.label_rule == NavType.LABEL_USE_SEARCH_FIELD:
            label = getattr(object, nt.search_field)
        elif nt.label_rule == NavType.LABEL_USE_GET_LABEL:
            label = object.get_label()
        else:
            label = unicode(object)
    except NavType.DoesNotExist:
        label = unicode(object)
    return label

def set_node_ordering(node, tree, parent):
    if parent:
        node.parent = parent
        sibling_nodes = NavNode.objects.filter(tree=tree, parent=node.parent)
    else:
        node.parent = None
        sibling_nodes = NavNode.objects.filter(tree=tree, parent__isnull=True)
    max_ordering = sibling_nodes.aggregate(max_ordering=Max('ordering'))['max_ordering'] or 0
    node.ordering = max_ordering + 1

def create_navigation_node(content_type, object, tree, parent):
    node = NavNode(tree=tree, label=get_object_label(content_type, object))
    #add it as last child of the selected node
    set_node_ordering(node, tree, parent)
    #associate with a content object
    node.content_type = content_type
    node.object_id = object.id if object else 0
    node.save()
    return node

class NavType(models.Model):
    """Define which ContentTypes can be inserted in the tree as content"""

    LABEL_USE_UNICODE = 0
    LABEL_USE_SEARCH_FIELD = 1
    LABEL_USE_GET_LABEL = 2

    LABEL_RULE_CHOICES = (
        (LABEL_USE_UNICODE, _(u'Use object unicode')),
        (LABEL_USE_SEARCH_FIELD, _(u'Use search field')),
        (LABEL_USE_GET_LABEL, _(u'Use get_label')),
    )

    content_type = models.ForeignKey(ContentType, unique=True, verbose_name=_(u'django model'))
    search_field = models.CharField(max_length=200, blank=True, default="", verbose_name=_(u'search field'))
    label_rule = models.IntegerField(verbose_name=_(u'How to generate the label'),
        choices=LABEL_RULE_CHOICES, default=LABEL_USE_UNICODE)

    def __unicode__(self):
        return self.content_type.app_label+'.'+self.content_type.model

    class Meta:
        verbose_name = _(u'navigable type')
        verbose_name_plural = _(u'navigable types')


class NavNode(models.Model):
    """
    A navigation node
    Part of the tree as child of his parent
    Point on a content_object
    """

    tree = models.ForeignKey(COOP_CMS_NAVTREE_CLASS, verbose_name=_("tree"))
    label = models.CharField(max_length=200, verbose_name=_("label"))
    parent = models.ForeignKey("NavNode", blank=True, null=True, default=0, verbose_name=_("parent"))
    ordering = models.PositiveIntegerField(_("ordering"), default=0)

    #generic relation
    content_type = models.ForeignKey(ContentType, verbose_name=_("content_type"), blank=True, null=True)
    object_id = models.PositiveIntegerField(verbose_name=_("object id"), blank=True, null=True)
    content_object = generic.GenericForeignKey('content_type', 'object_id')
    in_navigation = models.BooleanField(_("in navigation"), default=True)

    def get_absolute_url(self):
        if self.content_object:
            return self.content_object.get_absolute_url()
        return None

    def get_content_name(self):
        return self.content_type.model_class()._meta.verbose_name

    def __unicode__(self):
        return self.label

    class Meta:
        verbose_name = _(u'navigation node')
        verbose_name_plural = _(u'navigation nodes')
        #unique_together = ('content_type', 'object_id')

    def get_children(self, in_navigation=None):
        nodes = NavNode.objects.filter(parent=self).order_by("ordering")
        if in_navigation != None:
            nodes = nodes.filter(in_navigation=in_navigation)
        return nodes

    def has_children(self):
        return self.get_children(True).count()

    def get_siblings(self, in_navigation=None):
        nodes = NavNode.objects.filter(parent=self.parent).order_by("ordering")
        if in_navigation != None:
            nodes = nodes.filter(in_navigation=in_navigation)
        return nodes

    def get_progeny(self, level=0):
        progeny = []
        progeny.append((self, level))
        for child in NavNode.objects.filter(parent=self).order_by("ordering"):
            progeny.extend(child.get_progeny(level+1))
        return progeny

    def as_jstree(self):
        url = self.get_absolute_url()
        label = escape(self.label)
        if url == None:
            li_content = u'<a>{0}</a>'.format(label)
        else:
            li_content = u'<a href="{0}">{1}</a>'.format(url, label)

        children_li = [child.as_jstree() for child in self.get_children()]

        return u'<li id="node_{0}" rel={3}>{1}<ul>{2}</ul></li>'.format(
            self.id, li_content, u''.join(children_li), "in_nav" if self.in_navigation else "out_nav"
        )

    def _get_li_content(self, li_template):
        if li_template:
            t = li_template if hasattr(li_template, 'render') else get_template(li_template)
            return t.render(Context({'node': self}))
        else:
            url = self.get_absolute_url()
            if url == None:
                return u'<a>{0}</a>'.format(self.label)
            else:
                return u'<a href="{0}">{1}</a>'.format(url, self.label)

    def _get_ul_format(self, ul_template):
        if ul_template:
            t = ul_template if hasattr(ul_template, 'render') else get_template(ul_template)
            return t.render(Context({'node': self}))
        else:
            return u'<ul>{0}</ul>'

    def _get_li_args(self, li_args):
        if li_args:
            t = li_args if hasattr(li_args, 'render') else get_template(li_args)
            return t.render(Context({'node': self}))
        else:
            return u''

    def as_navigation(self, li_template=None, css_class="", ul_template=None, li_args=None):
        #Display the node and his children as nested ul and li html tags.
        #li_template is a custom template that can be passed

        if not self.in_navigation:
            return ""

        children_li = [child.as_navigation(li_template) for child in self.get_children(in_navigation=True)]
        ul_format = self._get_ul_format(ul_template)
        children_html = ul_format.format(u''.join(children_li)) if children_li else ""
        args = self._get_li_args(li_args)
        if args:
            css_class = " "+args
        return u'<li{0}>{1}{2}</li>'.format(css_class, self._get_li_content(li_template), children_html)

    def as_breadcrumb(self, li_template=None, css_class=""):
        html = self.parent.as_breadcrumb(li_template) if self.parent else u""
        return html + u'<li{0}>{1}</li>'.format(css_class, self._get_li_content(li_template))

    def children_as_navigation(self, li_template=None, css_class=""):
        children_li = [u'<li{0}>{1}</li>'.format(css_class, child._get_li_content(li_template))
            for child in self.get_children(in_navigation=True)]
        return  u''.join(children_li)

    def siblings_as_navigation(self, li_template=None, css_class=""):
        siblings_li = [u'<li{0}>{1}</li>'.format(css_class, sibling._get_li_content(li_template))
            for sibling in self.get_siblings(in_navigation=True)]
        return  u''.join(siblings_li)

    def check_new_navigation_parent(self, parent_id):
        if parent_id == self.id:
            raise ValidationError(_(u'A node can not be its own parent'))

        if parent_id:
            cur_node = NavNode.objects.get(id=parent_id)
            while cur_node:
                if cur_node.id == self.id:
                    raise ValidationError(_(u'A node can not be child of its own child'))
                cur_node = cur_node.parent


class BaseNavTree(models.Model):
    last_update = models.DateTimeField(auto_now=True)
    name = models.CharField(_(u'name'), max_length=100, db_index=True, unique=True, default='default')
    types = models.ManyToManyField('coop_cms.NavType', blank=True)

    def __unicode__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('navigation_tree', args=[self.id])

    def get_root_nodes(self):
        return NavNode.objects.filter(tree=self, parent__isnull=True).order_by("ordering")

    class Meta:
        verbose_name = _(u'Navigation tree')
        verbose_name_plural = _(u'Navigation trees')
        abstract = True

#content_cleaner = html_cleaner.HTMLCleaner(
#    allow_tags=['a', 'img', 'p', 'br', 'b', 'i', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6',
#        'sup', 'pre', 'ul', 'li', 'ol', 'table', 'th', 'tr', 'td', 'tbody', 'span', 'div',
#        'strong','u','dd','dt','dl','hr','em','font','iframe','object','param','embed'],
#        #iframe is a security risk but needed until we find how to integrate oembed
#    allow_attrs_for_tag={'a': ['href', 'target'], 'img': ['src', 'alt']}
#)
#title_cleaner = html_cleaner.HTMLCleaner(allow_tags=['br','span','em','i','strong','b','font','u'])

class ArticleCategory(models.Model):
    name = models.CharField(_(u'name'), max_length=100)
    slug = AutoSlugField(populate_from='name', max_length=100, unique=True)
    ordering = models.IntegerField(_(u'ordering'), default=0)
    in_rss = models.BooleanField(_(u'in rss'), default=False,
        help_text=_(u"The articles of this category will be listed in the main rss feed"))

    def __unicode__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('coop_cms_articles_category', args=[self.slug])

    class Meta:
        verbose_name = _(u'article category')
        verbose_name_plural = _(u'article categories')


class BaseNavigable(TimeStampedModel):

    class Meta:
        abstract = True

    def _get_navigation_parent(self):
        ct = ContentType.objects.get_for_model(self.__class__)
        nodes = NavNode.objects.filter(object_id=self.id, content_type=ct)
        if nodes.count():
            node = nodes[0]
            if node.parent:
                return node.parent.id
            else:
                return -node.tree.id
        else:
            return None

    def _set_navigation_parent(self, value):
        ct = ContentType.objects.get_for_model(self.__class__)
        if value != None:
            if value < 0:
                tree_id = -value
                tree = get_navtree_class().objects.get(id=tree_id)
                parent = None
            else:
                parent = NavNode.objects.get(id=value)
                tree = parent.tree

            create_navigation_node(ct, self, tree, parent)

    navigation_parent = property(_get_navigation_parent, _set_navigation_parent,
        doc=_("set the parent in navigation."))

    def save(self, do_not_create_nav=False, *args, **kwargs):
        ret = super(BaseNavigable, self).save(*args, **kwargs)
        if not do_not_create_nav:
            parent_id = getattr(self, '_navigation_parent', None)
            if parent_id != None:
                self.navigation_parent = parent_id
        return ret

class BaseArticle(BaseNavigable):
    """An article : static page, blog item, ..."""

    DRAFT = 0
    PUBLISHED = 1

    PUBLICATION_STATUS = (
        (DRAFT, _(u'Draft')),
        (PUBLISHED, _(u'Published')),
    )

    def get_logo_folder(self, filename):
        try:
            img_root = settings.CMS_ARTICLE_LOGO_FOLDER
        except AttributeError:
            img_root = 'cms_logos'
        return u'{0}/{1}/{2}'.format(img_root, self.id, filename)

    #slug = AutoSlugField(populate_from='title', max_length=100, unique=True)
    slug = models.CharField(max_length=100, unique=True, db_index=True, blank=False)
    #title = HTMLField(title_cleaner, verbose_name=_(u'title'), default=_('Page title'))
    #content = HTMLField(content_cleaner, verbose_name=_(u'content'), default=_('Page content'))
    title = models.TextField(_(u'title'), default='', blank=True)
    content = models.TextField(_(u'content'), default='', blank=True)
    publication = models.IntegerField(_(u'publication'), choices=PUBLICATION_STATUS, default=PUBLISHED)
    template = models.CharField(_(u'template'), max_length=200, default='', blank=True)
    logo = models.ImageField(upload_to=get_logo_folder, blank=True, null=True, default='')
    temp_logo = models.ImageField(upload_to=get_logo_folder, blank=True, null=True, default='')
    summary = models.TextField(_(u'Summary'), blank=True, default='')
    category = models.ForeignKey(ArticleCategory, verbose_name=_(u'Category'), blank=True, null=True, default=None, related_name="%(app_label)s_%(class)s_rel")
    in_newsletter = models.BooleanField(_(u'In newsletter'), default=True, help_text=_(u'Make this article available for newsletters.'))
    homepage_for_site = models.ForeignKey(Site, verbose_name=_(u'Homepage for site'), blank=True, null=True, default=None, related_name="homepage_article")
    headline = models.BooleanField(_(u"Headline"), default=False, help_text=_(u'Make this article appear on the home page'))
    publication_date = models.DateTimeField(_(u"Publication date"), default=datetime.now())
    sites = models.ManyToManyField(Site, verbose_name=_(u'site'), default=settings.SITE_ID)
    
    @property
    def is_homepage(self):
        if self.homepage_for_site:
            return self.homepage_for_site.id == settings.SITE_ID
        return False
    
    def next_in_category(self):
        if self.category:
            try:
                return get_article_class().objects.filter(category=self.category,
                    publication_date__gt=self.publication_date).order_by('publication_date')[0]
            except IndexError:
                pass
        
    def previous_in_category(self):
        if self.category:
            try:
                return get_article_class().objects.filter(category=self.category,
                    publication_date__lt=self.publication_date).order_by('-publication_date')[0]
            except IndexError:
                pass

    def logo_thumbnail(self, temp=False, logo_size=None):
        logo = self.temp_logo if (temp and self.temp_logo) else self.logo
        size = logo_size or get_article_logo_size(self)
        if logo:
            file = logo.file
        else:
            file = self._get_default_logo()
        try:
            return sorl_thumbnail.backend.get_thumbnail(file, size, crop='center')
        except Exception, msg:
            return file

    def _get_default_logo(self):
        #copy from static to media in order to use sorl thumbnail without raising a suspicious operation
        filename = get_default_logo()
        media_filename = os.path.normpath(settings.MEDIA_ROOT + '/coop_cms/' + filename)
        if not os.path.exists(media_filename):
            dir = os.path.dirname(media_filename)
            if not os.path.exists(dir):
                os.makedirs(dir)
            static_filename = finders.find(filename)
            shutil.copyfile(static_filename, media_filename)
        return File(open(media_filename, 'r'))

    def logo_list_display(self):
        if self.logo:
            thumb = default.backend.get_thumbnail(self.logo.file, ADMIN_THUMBS_SIZE)
            return '<img width="%s" src="%s" />' % (thumb.width, thumb.url)
        else:
            return _(u"No Image")
    logo_list_display.short_description = _(u"logo")
    logo_list_display.allow_tags = True

    class Meta:
        verbose_name = _(u"article")
        verbose_name_plural = _(u"articles")
        abstract = True

    def __unicode__(self):
        return self.title


    def save(self, *args, **kwargs):
        #autoslug localized title for creating locale_slugs
        if (not self.title) and (not self.slug):
            raise Exception(u"coop_cms.Article: slug can not be empty")
            
        if is_localized():
            from modeltranslation.utils import build_localized_fieldname
            for (lang_code, lang_name) in settings.LANGUAGES:
                loc_title_var = build_localized_fieldname('title', lang_code)
                locale_title = getattr(self, loc_title_var, '')
            
                loc_slug_var = build_localized_fieldname('slug', lang_code)
                locale_slug = getattr(self, loc_slug_var, '')
                
                if locale_title and not locale_slug:
                    slug = self.get_unique_slug(loc_slug_var, locale_title)
                    setattr(self, loc_slug_var, slug)
        else:
            if not self.slug:
                self.slug = self.get_unique_slug('slug', self.title)
        
        is_new = not bool(self.id)
        ret = super(BaseArticle, self).save(*args, **kwargs)
        
        if is_new:
            site = Site.objects.get(id=settings.SITE_ID)
            self.sites.add(site)
            ret = super(BaseArticle, self).save(do_not_create_nav=True)
        
        if self.homepage_for_site and (self.homepage_for_site.id == settings.SITE_ID):
            for a in get_article_class().objects.filter(homepage_for_site__id=settings.SITE_ID).exclude(id=self.id):
                a.homepage_for_site = None
                a.save()
        
        return ret
    
    def get_unique_slug(self, slug_field, title):
        slug = slugify(title)
        next, origin_slug = 2, slug
        while True:
            #Check that this slug doesn't already exist
            #The slug must be unique for all sites
            Article = get_article_class()
            try:
                attrs = {slug_field: slug}
                if self.id:
                    Article.objects.get(Q(**attrs) & ~Q(id=self.id))
                else:
                    Article.objects.get(**attrs)
                #oups the slug is already used: change it and try again
                slug = u"{0}{1}".format(origin_slug, next)
                next += 1
            except Article.DoesNotExist:
                break #Ok this slug is not used: break the loop and return
        return slug
        
    def template_name(self):
        possible_templates = get_article_templates(self, None)
        for (template, name) in possible_templates:
            if template == self.template:
                return name
        return u"?"

    def get_label(self):
        return self.title

    def get_absolute_url(self):
        return reverse('coop_cms_view_article', args=[self.slug])

    def get_edit_url(self):
        return reverse('coop_cms_edit_article', args=[self.slug])

    def get_cancel_url(self):
        return reverse('coop_cms_cancel_edit_article', args=[self.slug])

    def get_publish_url(self):
        return reverse('coop_cms_publish_article', args=[self.slug])

    def _can_change(self, user):
        ct = ContentType.objects.get_for_model(get_article_class())
        perm = '{0}.change_{1}'.format(ct.app_label, ct.model)
        return user.has_perm(perm)

    def can_view_article(self, user):
        if self.publication != BaseArticle.PUBLISHED:
            return self.can_edit_article(user)
        else:
            return True

    def can_edit_article(self, user):
        return self._can_change(user)

    def can_publish_article(self, user):
        return self._can_change(user)

class Link(BaseNavigable):
    """Link to a given url"""
    title = models.CharField(_(u'Title'), max_length=200, default=_(u"title"))
    url = models.CharField(_(u'URL'), max_length=200)

    def get_absolute_url(self):
        if is_localized():
            scheme, netloc, path, params, query, fragment = urlparse.urlparse(self.url)
            if not scheme:
                #the urls doesn't starts with http://, so it's a url managed by the site
                from localeurl.utils import locale_path
                locale = translation.get_language()                
                return locale_path(self.url, locale)
        return self.url

    def get_label(self):
        scheme, netloc, path, params, query, fragment = urlparse.urlparse(self.url)
        if scheme:
            return u"{0}{1}".format(netloc, path)
        return self.url

    def __unicode__(self):
        return self.title

    class Meta:
        verbose_name = _(u"link")
        verbose_name_plural = _(u"links")

def get_img_folder(instance, filename):
    try:
        img_root = settings.IMAGE_FOLDER
    except AttributeError:
        img_root = 'img'

    return u'{0}/{1}/{2}'.format(img_root,
        instance.created.strftime('%Y%d%m%H%M%S'), filename)

class Media(TimeStampedModel):
    name = models.CharField(_('name'), max_length=200, blank=True, default='')

    def __unicode__(self):
        return self.name

    class Meta:
        abstract = True

class Image(Media):
    file = models.ImageField(_('file'), upload_to=get_img_folder)

    def as_thumbnail(self):
        try:
            return sorl_thumbnail.backend.get_thumbnail(self.file.file, "64x64", crop='center')
        except Exception, msg:
            return self.file
        

    def get_absolute_url(self):
        return self.file.url

    class Meta:
        verbose_name = _(u'image')
        verbose_name_plural = _(u'images')

class Document(Media):
    def get_doc_folder(self, filename):
        if not self.is_private:
            try:
                doc_root = settings.DOCUMENT_FOLDER
            except AttributeError:
                doc_root = 'documents/public'
        else:
            try:
                doc_root = settings.PRIVATE_DOCUMENT_FOLDER
            except AttributeError:
                doc_root = 'documents/private'

        filename = os.path.basename(filename)
        #This is required for x-sendfile
        name, ext = os.path.splitext(filename)
        filename = slugify(name) + ext
        
        return u'{0}/{1}/{2}'.format(doc_root,
            self.created.strftime('%Y%d%m%H%M%S'), filename)

    file = models.FileField(_('file'), upload_to=get_doc_folder)
    is_private = models.BooleanField(_('is private'), default=False,
        help_text=_(u"Check this if you do not want to publish this document to all users"))
    category = models.ForeignKey(ArticleCategory, blank=True, null=True, default=None, verbose_name=_(u'category'))

    def can_download_file(self, user):
        return user.is_authenticated()

    def get_download_url(self):
        if self.is_private:
            return reverse('coop_cms_download_doc', args=[self.id])
        else:
            return self.file.url

    def get_ico_url(self, icotype):
        root, ext = os.path.splitext(self.file.name)
        ext = ext[1:].lower()  # remove leading dot
        if ext in ('pdf', 'doc', 'docx', 'xls', 'xlsx', 'txt', 'png', 'jpg', 'gif', \
                    'ppt', 'pps', 'mp3', 'ogg', 'html', 'rtf', 'zip', 'avi', \
                    'mov', 'mp4', ):
            return settings.STATIC_URL + u'img/filetypes/' + icotype + '/{0}.png'.format(ext)
        else:
            return settings.STATIC_URL + u'img/filetypes/' + icotype + '/default.png'

    def get_block_url(self):
        return self.get_ico_url('bloc')

    def get_fileicon_url(self):
        return self.get_ico_url('icon')

    class Meta:
        verbose_name = _(u'document')
        verbose_name_plural = _(u'documents')

class PieceOfHtml(models.Model):
    div_id = models.CharField(verbose_name=_(u"identifier"), max_length=100, db_index=True)
    #content = HTMLField(content_cleaner, verbose_name=_(u"content"), default="", blank=True)
    content = models.TextField(_(u"content"), default="", blank=True)

    def __unicode__(self):
        return self.div_id

    class Meta:
        verbose_name = _(u'piece of HTML')
        verbose_name_plural = _(u'pieces of HTML')

#delete node when content object is deleted
def remove_from_navigation(sender, instance, **kwargs):
    if hasattr(instance, 'id'):
        try:
            ct = ContentType.objects.get_for_model(instance)
            node = NavNode.objects.get(content_type=ct, object_id=instance.id)
            node.delete()
        except (NavNode.DoesNotExist, ContentType.DoesNotExist):
            pass
pre_delete.connect(remove_from_navigation)

class NewsletterItem(models.Model):
    content_type = models.ForeignKey(ContentType, verbose_name=_("content_type"))
    object_id = models.PositiveIntegerField(verbose_name=_("object id"))
    content_object = generic.GenericForeignKey('content_type', 'object_id')
    ordering = models.IntegerField(verbose_name=_("ordering"), default=0)

    class Meta:
        unique_together = (("content_type", "object_id"),)
        verbose_name = _(u'newsletter item')
        verbose_name_plural = _(u'newsletter items')
        ordering = ['ordering']

    def __unicode__(self):
        return u'{0}: {1}'.format(self.content_type, self.content_object)

#delete item when content object is deleted
def on_delete_newsletterable_item(sender, instance, **kwargs):
    if hasattr(instance, 'id'):
        try:
            ct = ContentType.objects.get_for_model(instance)
            item = NewsletterItem.objects.get(content_type=ct, object_id=instance.id)
            item.delete()
        except (NewsletterItem.DoesNotExist, ContentType.DoesNotExist):
            pass
pre_delete.connect(on_delete_newsletterable_item)

def create_newsletter_item(instance):
    ct = ContentType.objects.get_for_model(instance)
    if getattr(instance, 'in_newsletter', True):
        #Create a newsletter item automatically
        #An optional 'in_newsletter' field can skip the automatic creation if set to False
        return NewsletterItem.objects.get_or_create(content_type=ct, object_id=instance.id)
    elif hasattr(instance, 'in_newsletter'):
        #If 'in_newsletter' field existe and is False
        #We delete the Item if exists
        try:
            item = NewsletterItem.objects.get(content_type=ct, object_id=instance.id)
            item.delete()
            return None, True
        except NewsletterItem.DoesNotExist:
            return None, False

#create automatically a newsletter item for every objects configured as newsletter_item
def on_create_newsletterable_instance(sender, instance, created, raw, **kwargs):
    if sender in get_newsletter_item_classes():
        create_newsletter_item(instance)
post_save.connect(on_create_newsletterable_instance)

class Newsletter(models.Model):
    subject = models.CharField(max_length=200, verbose_name=_(u'subject'), blank=True, default="")
    #content = HTMLField(content_cleaner, verbose_name=_(u"content"), default="<br>", blank=True)
    content = models.TextField(_(u"content"), default="<br>", blank=True)
    items = models.ManyToManyField(NewsletterItem, blank=True)
    template = models.CharField(_(u'template'), max_length=200, default='', blank=True)
    site = models.ForeignKey(Site, verbose_name=_(u'site'), default=settings.SITE_ID)

    def get_items(self):
        return [item.content_object for item in self.items.all()]

    def get_items_by_category(self):
        items = self.get_items()
        def sort_by_category(item):
            category = getattr(item, 'category', None)
            if category:
                return category.ordering
            return 0
        items.sort(key=sort_by_category)
        return items


    def can_edit_newsletter(self, user):
        return user.has_perm('coop_cms.change_newsletter')
        
    def get_site_prefix(self):
        return "http://{0}".format(self.site.domain)

    def get_absolute_url(self):
        return reverse('coop_cms_view_newsletter', args=[self.id])

    def get_edit_url(self):
        return reverse('coop_cms_edit_newsletter', args=[self.id])

    def get_template_name(self):
        template = self.template
        if not template:
            self = 'coop_cms/newsletter.html'
        return template

    def __unicode__(self):
        return self.subject

    class Meta:
        verbose_name = _(u'newsletter')
        verbose_name_plural = _(u'newsletters')

class NewsletterSending(models.Model):

    newsletter = models.ForeignKey(Newsletter)

    scheduling_dt = models.DateTimeField(_(u"scheduling date"), blank=True, default=None, null=True)
    sending_dt = models.DateTimeField(_(u"sending date"), blank=True, default=None, null=True)

    def __unicode__(self):
        return self.newsletter.subject

    class Meta:
        verbose_name = _(u'newsletter sending')
        verbose_name_plural = _(u'newsletter sendings')

class Alias(models.Model):
    
    path = models.CharField(max_length=200)
    redirect_url = models.CharField(max_length=200, default="", blank=True)
    
    class Meta:
        verbose_name = _(u'Alias')
        verbose_name_plural = _(u'Aliases')
    
    def get_absolute_url(self):
        return reverse('coop_cms_view_article', args=[self.path])
    
    def __unicode__(self):
        return self.path


    


#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    from setuptools import setup, find_packages
except ImportError:
    import ez_setup
    ez_setup.use_setuptools()
    from setuptools import setup, find_packages

VERSION = __import__('coop_cms').__version__

import os


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = 'apidev-coop_cms',
    version = VERSION,
    description = 'Small CMS built around a tree navigation open to any django models',
    packages = ['coop_cms',
                'coop_cms.apps',
                'coop_cms.apps.basic_cms',
                'coop_cms.apps.demo_cms',
                'coop_cms.management',
                'coop_cms.management.commands',
                'coop_cms.templatetags',
                'coop_cms.migrations',
                'coop_cms.apps.basic_cms.migrations',
                'coop_cms.apps.demo_cms.migrations',
                'coop_cms.apps.rss_sync',
                'coop_cms.apps.rss_sync.migrations',
                'coop_cms.apps.rss_sync.management',
                'coop_cms.apps.rss_sync.management.commands',
                'coop_cms.apps.test_app',
                ],
    include_package_data = True,
    author = 'Luc Jean',
    author_email = 'ljean@apidev.fr',
    license = 'BSD',
    zip_safe = False,
    install_requires = ['django-floppyforms==0.4.7',
                        'django-extensions==0.9',
                        'sorl-thumbnail==11.09',
                        'apidev-coop_colorbox==0.9.0',
                        'apidev-coop_bar==0.9.0',
                        'apidev-djaloha==0.9.0',
                        'django-pagination',
                        'feedparser',
                        #'django-filetransfers', #need to be installed manually from https://bitbucket.org/wkornewald/django-filetransfers
                        #'model_mommy', #---> ramène django 1.5 dans ses dépendances quand installé via coop_cms
                        ],
    long_description = open('README.rst').read(),
    url = 'https://github.com/ljean/coop_cms/',
    download_url = 'https://github.com/ljean/coop_cms/tarball/master',
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
        'Natural Language :: English',
        'Natural Language :: French',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
    ],

)




# -*- coding: utf-8 -*-

from django.contrib.syndication.views import Feed
from coop_cms.settings import get_article_class

class ArticleFeed(Feed):
    title = ""
    link = ""
    description = ""

    def items(self):
        Article = get_article_class()
        return Article.objects.filter(category__in_rss=True).order_by('-publication_date')

    def item_title(self, item):
        return item.title

    def item_description(self, item):
        return item.summary


# -*- coding: utf-8 -*-

from HTMLParser import HTMLParser
from re import sub
from sys import stderr
from traceback import print_exc
from django.core.mail import get_connection, EmailMultiAlternatives
from coop_cms.html2text import html2text
from django.conf import settings
from django.template.loader import get_template
from django.template import Context
from django.core.urlresolvers import reverse
from coop_cms.settings import get_article_class, is_localized, get_newsletter_context_callbacks
from coop_cms.models import BaseArticle, Alias
from django.utils.translation import get_language
from django.http import Http404, HttpResponsePermanentRedirect
from django.shortcuts import get_object_or_404
from bs4 import BeautifulSoup

class _DeHTMLParser(HTMLParser):
    def __init__(self):
        HTMLParser.__init__(self)
        self.__text = []

    def handle_data(self, data):
        text = data.strip()
        if len(text) > 0:
            text = sub('[ \t\r\n]+', ' ', text)
            self.__text.append(text + ' ')

    def handle_starttag(self, tag, attrs):
        if tag == 'p':
            self.__text.append('\n\n')
        elif tag == 'br':
            self.__text.append('\n')

    def handle_startendtag(self, tag, attrs):
        if tag == 'br':
            self.__text.append('\n\n')

    def text(self):
        return ''.join(self.__text).strip()


# copied from http://stackoverflow.com/a/3987802/117092
def dehtml(text):
    try:
        parser = _DeHTMLParser()
        parser.feed(text)
        parser.close()
        return parser.text()
    except:
        print_exc(file=stderr)
        return text

def make_links_absolute(html_content, newsletter=None):
    """replace all local url with site_prefixed url"""
    
    def make_abs(url):
        if url.startswith('..'):
            url = url[2:]
        while url.startswith('/..'):
            url = url[3:]
        if url.startswith('/'):
            url = '%s%s' % (site_prefix, url)
        return url
    
    site_prefix = newsletter.get_site_prefix() if newsletter else settings.COOP_CMS_SITE_PREFIX
    soup = BeautifulSoup(html_content)
    for a_tag in soup.find_all("a"):
        if a_tag.get("href", None):
            a_tag["href"] = make_abs(a_tag["href"])
    
    for img_tag in soup.find_all("img"):
        if img_tag.get("src", None):
            img_tag["src"] = make_abs(img_tag["src"])
    
    return soup.prettify()
    
#def make_links_absolute(html_content, newsletter=None):
#    """replace all local url with absolute url"""
#    import re
#    #regex = """<.*(?P<tag>href|src)\s*=\s*["'](?P<url>.+?)["'].*>"""
#    #regex = """<.*href|src\s*=\s*["'](?P<url>.+?)["'].*>"""
#    
#    site_prefix = newsletter.get_site_prefix() if newsletter else settings.COOP_CMS_SITE_PREFIX
#    
#    def make_abs(match):
#        #Thank you : http://www.gawel.org/howtos/python-re-sub
#        start = match.group('start')
#        url = match.group('url')
#        
#        if url.startswith('..'):
#            url = url[2:]
#        while url.startswith('/..'):
#            url = url[3:]
#        if url.startswith('/'):
#            url = '%s%s' % (site_prefix, url)
#        end = match.group('end')
#        
#        print start, url, end
#        
#        return start + url + end
#    
#    #a_pattern = re.compile(ur'(?P<start>.*?href=")(?P<url>\S+)(?P<end>".*?)')
#    a_pattern = re.compile(r"""(?P<start>.*?href=")(?P<url>\S+)(?P<end>".*?)""")
#    html_content = a_pattern.sub(make_abs, html_content)
#    #raise
#    
#    img_pattern = re.compile(r'(?P<start>.*?src=")(?P<url>\S+)(?P<end>".*?)')
#    html_content = img_pattern.sub(make_abs, html_content)
#
#    return html_content
    
def send_newsletter(newsletter, dests):

    t = get_template(newsletter.get_template_name())
    context_dict = {
        'title': newsletter.subject, 'newsletter': newsletter, 'by_email': True,
        'SITE_PREFIX': settings.COOP_CMS_SITE_PREFIX,
        'MEDIA_URL': settings.MEDIA_URL, 'STATIC_URL': settings.STATIC_URL,
    }
    
    for callback in get_newsletter_context_callbacks():
        d = callback(newsletter)
        if d:
            context_dict.update(d)

    html_text = t.render(Context(context_dict))
    html_text = make_links_absolute(html_text, newsletter)
    
    emails = []
    connection = get_connection()
    from_email = settings.COOP_CMS_FROM_EMAIL
    reply_to = getattr(settings, 'COOP_CMS_REPLY_TO', None)
    headers = {'Reply-To': reply_to} if reply_to else None

    for addr in dests:
        text = html2text(html_text)
        email = EmailMultiAlternatives(newsletter.subject, text, from_email, [addr], headers=headers)
        email.attach_alternative(html_text, "text/html")
        emails.append(email)
    return connection.send_messages(emails)

def get_article_slug(*args, **kwargs):
    slug = reverse(*args, **kwargs)
    if 'localeurl' in settings.INSTALLED_APPS:
        #If localeurl is installed reverse is patched
        #We must remove the lang prefix
        from localeurl.utils import strip_path
        lang, slug = strip_path(slug)
    return slug.strip('/')

def get_article(slug, current_lang=None, force_lang=None, **kwargs):
    Article = get_article_class()
    try:
        return Article.objects.get(slug=slug, **kwargs)
    except Article.DoesNotExist:
        #if modeltranslation is installed,
        #if no article correspond to the current language article
        #try to look for slug in default language
        if is_localized():
            from modeltranslation import settings as mt_settings
            default_lang = mt_settings.DEFAULT_LANGUAGE
            lang = force_lang
            if not lang:
                current_lang = get_language()
                if current_lang != default_lang:
                    lang = default_lang
            if lang:
                kwargs.update({'slug_{0}'.format(lang): slug})
                return Article.objects.get(**kwargs)
        raise #re-raise previous error

def get_article_or_404(slug, **kwargs):
    Article = get_article_class()
    try:
        return get_article(slug, **kwargs)
    except Article.DoesNotExist:
        raise Http404

def get_headlines(article):
    Article = get_article_class()
    if article.is_homepage:
        return Article.objects.filter(headline=True, publication=BaseArticle.PUBLISHED).order_by("-publication_date")
    return Article.objects.none()
    
def redirect_if_alias(path):
    alias = get_object_or_404(Alias, path=path)
    if alias.redirect_url:
        return HttpResponsePermanentRedirect(alias.redirect_url)
    else:
        raise Http404